<?php
/**
 * @package    Essentials for VirtueMart
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2024 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

use \Joomla\CMS\Factory;

/**
 * Essentials for VirtueMart Component Package Script
 * 
 * @since   1.0
 */
class com_vmessentialsScript
{
	protected static $tableColumns = [];
	
	/**
	 * Called on installation
	 *
	 * @param   Joomla\CMS\Installer\Adapter\PackageAdapter  $adapter  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function install($adapter)
	{
		return true;
	}

	/**
	 * Called on update
	 *
	 * @param   Joomla\CMS\Installer\Adapter\PackageAdapter  $adapter  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function update($adapter)
	{
		$db = Factory::getDbo();
		
		$this->setDefaultValueToVarcharColumns('#__vmessentials_wishlist');
		$this->setDefaultValueToIntegerColumns('#__vmessentials_wishlist');

		return true;
	}

	/**
	 * Called before any type of action
	 *
	 * @param     string                                         $route      Which action is happening (install|uninstall|discover_install)
	 * @param     Joomla\CMS\Installer\Adapter\PackageAdapter    $adapter    The object responsible for running this script
	 *
	 * @return    boolean                         True on success
	 */
	public function preflight($route, $adapter)
	{
		if ($route === 'update')
		{
			$db = Factory::getDbo();
			
			// Alter table if needed.
		}

		return true;
	}

	/**
	 * Called after any type of action
	 *
	 * @param     string                                         $route      Which action is happening (install|uninstall|discover_install)
	 * @param     Joomla\CMS\Installer\Adapter\PackageAdapter    $adapter    The object responsible for running this script
	 *
	 * @return    boolean                         True on success
	 */
	public function postflight($route, $adapter)
	{
		return true;
	}
	
	/**
	* Method to set a default value to non-nullable varchar columns.
	*
	* @param   string  $table  The name of the table.
	*
	* @return void
	*/
	private function setDefaultValueToVarcharColumns($table)
	{
		$db      = Factory::getDbo();
		$columns = $this->getTableColumns($table);
		
		foreach ($columns as $column => $data)
		{
			if (!property_exists($data, 'Type') || !property_exists($data, 'Default') || !property_exists($data, 'Null'))
			{
				continue;
			}
			
			if (stripos($data->Type, 'varchar') === 0 && $data->Null === 'NO' && $data->Default === null)
			{
				$db->setQuery('ALTER TABLE ' . $db->quoteName($table) . ' MODIFY COLUMN ' . $db->quoteName($column) . ' ' . $data->Type . ' NOT NULL DEFAULT ' . $db->quote(''));
				$db->execute();
			}
		}
	}
	
	/**
	* Method to set a default value to non-nullable int or tinyint columns.
	*
	* @param   string   $table         The name of the table.
	* @param   integer  $defaultValue  The default value.
	*
	* @return void
	*/
	private function setDefaultValueToIntegerColumns($table, $defaultValue = 0)
	{
		$db      = Factory::getDbo();
		$columns = $this->getTableColumns($table);
		
		foreach ($columns as $column => $data)
		{
			if (!property_exists($data, 'Type') || !property_exists($data, 'Default') || !property_exists($data, 'Null'))
			{
				continue;
			}
			
			if ($column === 'id' && $data->Default == '0')
			{
				$query = $db->getQuery(true);
				
				$query->select('MAX(id)')
					->from($db->quoteName($table));
				
				$db->setQuery($query);
				$maxId = $db->loadResult();
				
				$db->setQuery('ALTER TABLE ' . $db->quoteName($table) . ' CHANGE COLUMN ' . $db->quoteName('id') . ' ' . $db->quoteName('id') . ' int(11) UNSIGNED NOT NULL AUTO_INCREMENT, AUTO_INCREMENT = ' . ($maxId + 1));
				$db->execute();
			}
			elseif ((stripos($data->Type, 'int') === 0 || stripos($data->Type, 'tinyint') === 0) && $data->Null === 'NO' && $data->Default === null && $column !== 'id')
			{
				$db->setQuery('ALTER TABLE ' . $db->quoteName($table) . ' MODIFY COLUMN ' . $db->quoteName($column) . ' ' . $data->Type . ' NOT NULL DEFAULT ' . $db->quote($defaultValue));
				$db->execute();
			}
		}
	}
	
	/**
	* Check if a column exists in a database table.
	*
	* @param    string    $table   The name of the table.
	* @param    string    $column  The name of the column.
	*
	* @return   boolean
	*/
	private function columnExists($table, $column)
	{
		$columns     = $this->getTableColumns($table);
		$columnNames = array_keys($columns);
		
		return in_array($column, $columnNames);
	}
	
	/**
	* Check if a column has unsigned attribute in a database table.
	*
	* @param    string    $table   The name of the table.
	* @param    string    $column  The name of the column.
	*
	* @return
	*/
	private function isTableColumnUnsigned($table, $column)
	{
		$columns = $this->getTableColumns($table);
		
		if (array_key_exists($column, $columns))
		{
			$columnData = $columns[$column];
			
			if (property_exists($columnData, 'Type') && stripos($columnData->Type, 'unsigned') === false)
			{
				return false;
			}
		}
		
		return true;
	}
	
	/**
	* Method to get a list of columns of a database table.
	*
	* @param     string    $table  The name of the database table.
	*
	* @return    array
	*/
	private function getTableColumns($table)
	{
		if (!isset(self::$tableColumns[$table]))
		{
			$columns = Factory::getDbo()->getTableColumns($table, false);
			
			self::$tableColumns[$table] = !empty($columns) ? $columns : [];
		}
		
		return self::$tableColumns[$table];
	}
}
